# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Building 'Segment' Level Data Attributes"""

try:
    from typing import Any, Dict
except ImportError:
    pass  # Python2.7

from honeybee_ph import _base, phius, phi, site
from honeybee_ph_standards.sourcefactors import factors
from honeybee_ph_utils import enumerables
from honeybee_energy_ph.construction import thermal_bridge


class BuildingCategoryType(enumerables.CustomEnum):
    allowed = [
        "RESIDENTIAL",
        "NON-RESIDENTIAL"
    ]

    def __init__(self, _value=1):
        super(BuildingCategoryType, self).__init__()
        self.value = _value


class OccupancyType(enumerables.CustomEnum):
    allowed = [
        "RESIDENTIAL",  # 1
        "",  # 2
        "",  # 3
        "OFFICE/ADMINISTRATIVE BUILDING",  # 4
        "SCHOOL",  # 5
        "OTHER",  # 6
        "UNDEFINED/UNFINISHED",  # 7
    ]

    def __init__(self, _value=1):
        super(OccupancyType, self).__init__()
        self.value = _value


class SetPoints(_base._Base):
    def __init__(self):
        super(SetPoints, self).__init__()
        self.winter = 20.0
        self.summer = 25.0

    def to_dict(self):
        # type: () -> dict[str, float]
        d = {}

        d['winter'] = self.winter
        d['summer'] = self.summer

        return d

    @classmethod
    def from_dict(cls, _dict):
        # type: (dict[str, float]) -> SetPoints
        obj = cls()

        obj.winter = _dict.get('winter', 20.0)
        obj.summer = _dict.get('summer', 25.0)

        return obj

    def __copy__(self):
        # type: () -> SetPoints
        obj = SetPoints()
        obj.set_base_attrs_from_source(self)
        obj.winter = self.winter
        obj.summer = self.summer
        return obj

    def duplicate(self):
        # type: () -> SetPoints
        return self.__copy__()


class BldgSegment(_base._Base):

    def __init__(self):
        super(BldgSegment, self).__init__()
        self.display_name = 'Unnamed_Bldg_Segment'
        self.num_floor_levels = 1
        self.num_dwelling_units = 1
        self.site = site.Site()
        self.source_energy_factors = factors.FactorCollection('Source_Energy')
        self.co2e_factors = factors.FactorCollection('CO2')
        self.phius_certification = phius.PhiusCertification()
        self.phi_certification = phi.PhiCertification()
        self.set_points = SetPoints()
        self.thermal_bridges = {}

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = {}

        d['identifier'] = self.identifier
        d['name'] = self.display_name
        d['num_floor_levels'] = self.num_floor_levels
        d['num_dwelling_units'] = self.num_dwelling_units
        d['site'] = self.site.to_dict()
        d['source_energy_factors'] = self.source_energy_factors.to_dict()
        d['co2e_factors'] = self.co2e_factors.to_dict()
        d['phius_certification'] = self.phius_certification.to_dict()
        d['phi_certification'] = self.phi_certification.to_dict()
        d['set_points'] = self.set_points.to_dict()
        d['thermal_bridges'] = {}
        for tb in self.thermal_bridges.values():
            d['thermal_bridges'][str(tb.identifier)] = tb.to_dict()
        return d

    @classmethod
    def from_dict(cls, _dict):
        # type: (dict[str, Any]) -> BldgSegment
        obj = cls()

        obj.identifier = _dict.get('identifier')
        obj.display_name = _dict.get('name')
        obj.num_floor_levels = _dict.get('num_floor_levels')
        obj.num_dwelling_units = _dict.get('num_dwelling_units')
        obj.site = site.Site.from_dict(_dict.get('site', {}))
        obj.source_energy_factors = factors.FactorCollection.from_dict(
            _dict.get('source_energy_factors', {}))
        obj.co2e_factors = factors.FactorCollection.from_dict(
            _dict.get('co2e_factors', {}))
        obj.phius_certification = phius.PhiusCertification.from_dict(
            _dict.get('phius_certification', {}))
        obj.phi_certification = phi.PhiCertification.from_dict(
            _dict.get('phi_certification', {}))
        obj.set_points = SetPoints.from_dict(_dict.get('set_points', {}))
        for tb_dict in _dict['thermal_bridges'].values():
            tb_obj = thermal_bridge.PhThermalBridge.from_dict(tb_dict)
            obj.thermal_bridges[tb_obj.identifier] = tb_obj
        return obj

    def __copy__(self):
        # type () -> BldgSegment
        new_obj = BldgSegment()

        new_obj.identifier = self.identifier
        new_obj.display_name = self.display_name
        new_obj.num_floor_levels = self.num_floor_levels
        new_obj.num_dwelling_units = self.num_dwelling_units
        new_obj.site = self.site.duplicate()
        new_obj.source_energy_factors = self.source_energy_factors.duplicate()
        new_obj.co2e_factors = self.co2e_factors.duplicate()
        new_obj.phius_certification = self.phius_certification.duplicate()
        new_obj.phi_certification = self.phi_certification.duplicate()
        new_obj.set_points = self.set_points.duplicate()
        new_obj.thermal_bridges = {}
        for tb_k, tb_v in self.thermal_bridges.items():
            new_obj.thermal_bridges[tb_k] = tb_v

        return new_obj

    def duplicate(self):
        # type () -> BldgSegment
        return self.__copy__()



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Subclassing Honeybee-Energy | Load | Lighting"""

try:
    from typing import Any
except ImportError:
    # Python 2.7
    pass

from honeybee import properties
from honeybee_energy.load import lighting


class LightingPhProperties(object):
    """PH Properties Object for Honeybee-Energy Lighting"""

    def __init__(self, _host):
        self._host = _host
        self.id_num = 0

    def duplicate(self, new_host=None):
        # type: (LightingPhProperties, Any) -> LightingPhProperties
        _host = new_host or self._host
        new_properties_obj = self.__class__(_host)
        new_properties_obj.id_num = self.id_num

        return new_properties_obj

    def ToString(self):
        return self.__repr__()

    @property
    def host(self):
        return self._host

    def __repr__(self):
        return "LBT-Point3D Passive House Properties: [host: {}]".format(self.host.display_name)

    def to_dict(self, abridged=False):
        # type: (LightingPhProperties, bool) -> dict[str, dict]
        base = {"_PH": {}}
        base["_PH"]["type"] = "LightingPhProperties" if not abridged else "LightingPhPropertiesAbridged"

        return base

    @classmethod
    def from_dict(cls, data, host):
        # type: (dict, Any) -> LightingPhProperties
        assert data["type"] == "LightingPhProperties", "Expected LightingPhProperties. Got {}.".format(
            data["type"])

        new_prop = cls(host)
        new_prop.id_num = data.get("id_num", 0)

        return new_prop


class LightingProperties(properties._Properties):
    """Properties for HBE Lighting Objects"""

    def __repr__(self):
        return "LightingProperties: {!r}".format(self.host)

    def to_dict(self, abridged=False):
        # type: (LightingProperties, bool) -> dict
        """Convert properties to dictionary.

        Args:
            abridged: Boolean to note whether the full dictionary describing the
                object should be returned (False) or just an abridged version (True).
                Default: False.
        """
        base = {'type': 'LightingProperties'} if not abridged else \
            {'type': 'LightingPropertiesAbridged'}

        return base


class PH_Lighting(lighting.Lighting):
    __slots__ = ("_properties",)

    def __init__(self, *args, **kwargs):
        super(PH_Lighting, self).__init__(*args, **kwargs)
        self._properties = LightingProperties(self)

    @property
    def properties(self):
        return self._properties

    @classmethod
    def from_hb_lighting(cls, _hb_lighting):
        # type: (lighting) -> PH_Lighting
        new_ph_obj = cls(
            identifier=_hb_lighting.identifier,
            watts_per_area=_hb_lighting.watts_per_area,
            schedule=_hb_lighting.schedule,
            return_air_fraction=_hb_lighting.return_air_fraction,
            radiant_fraction=_hb_lighting.radiant_fraction,
            visible_fraction=_hb_lighting.visible_fraction,
        )

        return new_ph_obj

    def __setattr__(self, attr_name, value):
        # type: (PH_Lighting, str, Any) -> None
        """Override in order to intercept any attribute setting on the 
        base LBT object. Use the following syntax to catch any calls and 
        pass them to the PH .properties as approprate. Be sure to include the 
        super() at the end to also set the attr on the base LBT object.

        > if attr_name == "watts_per_area":
        >    self._properties.ph.watts_per_area = value
        >    ...
        > super(PH_Lighting, self).__setattr__(attr_name, value)
        """

        super(PH_Lighting, self).__setattr__(attr_name, value)
        return None

    def ToString(self):
        return self.__repr__()

    def __repr__(self):
        return "{}".format(self.__class__.__name__)

    def __copy__(self):
        # type: (PH_Lighting) -> PH_Lighting
        new_hb_obj = super(PH_Lighting, self).__copy__()
        new_ph_obj = PH_Lighting.from_hb_lighting(new_hb_obj)
        new_ph_obj._properties = self._properties
        return new_ph_obj

    def to_dict(self, abridged=False):
        # type: (PH_Lighting, bool) -> dict
        base = super(PH_Lighting, self).to_dict(abridged=abridged)
        base.update({"_properties": self.properties.to_dict(abridged)})
        return base



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""PHI Certification Settings Class"""

try:
    from typing import Optional
except ImportError:
    pass  # IronPython 2.7

from honeybee_ph_utils import enumerables
from honeybee_ph import _base


# -----------------------------------------------------------------------------


class PhiBuildingCategoryType(enumerables.CustomEnum):
    allowed = [
        "1-RESIDENTIAL BUILDING",
        "2-NON-RESIDENTIAL BUILDING",
    ]

    def __init__(self, _value=1):
        super(PhiBuildingCategoryType, self).__init__(_value)


class PhiBuildingUseType(enumerables.CustomEnum):
    allowed = [
        "",  # PHPP is so stupid. Who did the numbering?... sheesh...
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "10-DWELLING",
        "11-NURSING HOME / STUDENTS",
        "12-OTHER",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "20-OFFICE / ADMIN. BUILDING",
        "21-SCHOOL",
        "22-OTHER",
    ]

    def __init__(self, _value=1):
        super(PhiBuildingUseType, self).__init__(_value)

        # Quick double check cus' the numbering here is so stupid
        if self.value == "":
            raise Exception(
                "Error: PHPP's '_building_use_type' numbering is weird. Check the inputs is valid."
            )


class PhiIHGType(enumerables.CustomEnum):
    allowed = [
        "",
        "2-STANDARD",
        "3-PHPP CALCULATION ('IHG' WORKSHEET)",
        "4-PHPP CALCULATION ('IHG NON-RES' WORKSHEET)",
    ]

    def __init__(self, _value=1):
        super(PhiIHGType, self).__init__(_value)

        # Quick double check cus' the numbering here is so stupid
        if self.value == "":
            raise Exception(
                "Error: PHPP's '_building_use_type' numbering is weird. Check the inputs is valid."
            )


class PhiOccupancyType(enumerables.CustomEnum):
    allowed = [
        "1-STANDARD (ONLY FOR RESIDENTIAL BUILDINGS)",
        "2-USER DETERMINED",
    ]

    def __init__(self, _value=1):
        super(PhiOccupancyType, self).__init__(_value)


class PhiCertificationType(enumerables.CustomEnum):
    allowed = [
        "1-PASSIVE HOUSE",
        "2-ENERPHIT",
        "3-PHI LOW ENERGY BUILDING",
        "4-OTHER",
    ]

    def __init__(self, _value=1):
        super(PhiCertificationType, self).__init__(_value)


class PhiCertificationClass(enumerables.CustomEnum):
    allowed = [
        "1-CLASSIC",
        "2-PLUS",
        "3-PREMIUM",
    ]

    def __init__(self, _value=1):
        super(PhiCertificationClass, self).__init__(_value)


class PhiPrimaryEnergyType(enumerables.CustomEnum):
    allowed = [
        "1-PE (NON-RENEWABLE)",
        "2-PER (RENEWABLE)",
    ]

    def __init__(self, _value=1):
        super(PhiPrimaryEnergyType, self).__init__(_value)


class PhiEnerPHitType(enumerables.CustomEnum):
    allowed = [
        "1-COMPONENT METHOD",
        "2-ENERGY DEMAND METHOD",
    ]

    def __init__(self, _value=1):
        super(PhiEnerPHitType, self).__init__(_value)


class PhiRetrofitType(enumerables.CustomEnum):
    allowed = [
        "1-NEW BUILDING",
        "2-RETROFIT",
        "3-STEP-BY-STEP RETROFIT",
    ]

    def __init__(self, _value=1):
        super(PhiRetrofitType, self).__init__(_value)


# -----------------------------------------------------------------------------


class PhiCertification(_base._Base):

    def __init__(self):
        super(PhiCertification, self).__init__()
        self._building_category_type = PhiBuildingCategoryType("1-RESIDENTIAL BUILDING")
        self._building_use_type = PhiBuildingUseType("10-DWELLING")
        self._ihg_type = PhiIHGType("2-Standard")
        self._occupancy_type = PhiOccupancyType(
            "1-STANDARD (ONLY FOR RESIDENTIAL BUILDINGS)")

        self._certification_type = PhiCertificationType("1-PASSIVE HOUSE")
        self._certification_class = PhiCertificationClass("1-CLASSIC")
        self._primary_energy_type = PhiPrimaryEnergyType("2-PER (RENEWABLE)")
        self._enerphit_type = PhiEnerPHitType("2-ENERGY DEMAND METHOD")
        self._retrofit_type = PhiRetrofitType("1-NEW BUILDING")

    @property
    def building_category_type(self):
        return self._building_category_type

    @building_category_type.setter
    def building_category_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._building_category_type = PhiBuildingCategoryType(_input)

    @property
    def building_use_type(self):
        return self._building_use_type

    @building_use_type.setter
    def building_use_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._building_use_type = PhiBuildingUseType(_input)

    @property
    def occupancy_type(self):
        return self._occupancy_type

    @occupancy_type.setter
    def occupancy_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._occupancy_type = PhiOccupancyType(_input)

    @property
    def ihg_type(self):
        return self._ihg_type

    @ihg_type.setter
    def ihg_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._ihg_type = PhiIHGType(_input)

    @property
    def certification_type(self):
        return self._certification_type

    @certification_type.setter
    def certification_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._certification_type = PhiCertificationType(_input)

    @property
    def certification_class(self):
        return self._certification_class

    @certification_class.setter
    def certification_class(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._certification_class = PhiCertificationClass(_input)

    @property
    def primary_energy_type(self):
        return self._primary_energy_type

    @primary_energy_type.setter
    def primary_energy_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._primary_energy_type = PhiPrimaryEnergyType(_input)

    @property
    def enerphit_type(self):
        return self._enerphit_type

    @enerphit_type.setter
    def enerphit_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._enerphit_type = PhiEnerPHitType(_input)

    @property
    def retrofit_type(self):
        return self._retrofit_type

    @retrofit_type.setter
    def retrofit_type(self, _input):
        # type: (Optional[str]) -> None
        if _input:
            self._retrofit_type = PhiRetrofitType(_input)

    def __str__(self):
        return "{}()".format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)

    def to_dict(self):
        # type: () -> dict
        d = {}
        d['_building_category_type'] = self._building_category_type.to_dict()
        d['_building_use_type'] = self._building_use_type.to_dict()
        d['_ihg_type'] = self._ihg_type.to_dict()
        d['_occupancy_type'] = self._occupancy_type.to_dict()

        d['_certification_type'] = self._certification_type.to_dict()
        d['_certification_class'] = self._certification_class.to_dict()
        d['_primary_energy_type'] = self._primary_energy_type.to_dict()
        d['_enerphit_type'] = self._enerphit_type.to_dict()
        d['_retrofit_type'] = self._retrofit_type.to_dict()
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhiCertification
        new_obj = cls()

        new_obj._building_category_type = PhiBuildingCategoryType.from_dict(
            _input_dict['_building_category_type'])
        new_obj._building_use_type = PhiBuildingUseType.from_dict(
            _input_dict['_building_use_type'])
        new_obj._ihg_type = PhiIHGType.from_dict(
            _input_dict['_ihg_type'])
        new_obj._occupancy_type = PhiOccupancyType.from_dict(
            _input_dict['_occupancy_type'])

        new_obj._certification_type = PhiCertificationType.from_dict(
            _input_dict['_certification_type'])
        new_obj._certification_class = PhiCertificationClass.from_dict(
            _input_dict['_certification_class'])
        new_obj._primary_energy_type = PhiPrimaryEnergyType.from_dict(
            _input_dict['_primary_energy_type'])
        new_obj._enerphit_type = PhiEnerPHitType.from_dict(
            _input_dict['_enerphit_type'])
        new_obj._retrofit_type = PhiRetrofitType.from_dict(
            _input_dict['_retrofit_type'])
        return new_obj

    def __copy__(self):
        # type: () -> PhiCertification
        obj = PhiCertification()

        obj.set_base_attrs_from_source(self)
        obj._building_category_type = self._building_category_type
        obj._building_use_type = self._building_use_type
        obj._ihg_type = self._ihg_type
        obj._occupancy_type = self._occupancy_type
        obj._certification_type = self._certification_type
        obj._certification_class = self._certification_class
        obj._primary_energy_type = self._primary_energy_type
        obj._enerphit_type = self._enerphit_type
        obj._retrofit_type = self._retrofit_type

        return obj

    def duplicate(self):
        # type: () -> PhiCertification
        return self.__copy__()



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Passive House Service Hot Water Objects"""

try:
    from typing import Any, Union, List, Dict
except ImportError:
    pass  # IronPython

from ladybug_geometry.geometry3d.polyline import LineSegment3D

from honeybee_energy_ph.hvac import _base
from honeybee_ph_utils import enumerables


class UnknownPhHeaterTypeError(Exception):
    def __init__(self, _heater_types, _received_type):
        self.msg = 'Error: Unknown SHW Heater Type? Got: "{}" but only types: {} are allowed?'.format(
            _received_type, _heater_types)
        super(UnknownPhHeaterTypeError, self).__init__(self.msg)


# -- Piping ---------------------------------------------------------------------


class PhPipeSegment(_base._PhHVACBase):
    """A single pipe segment (linear) with geometry and a diameter"""

    def __init__(self, _geom, _diameter=0.0127, _insul_thickness=0.0, _insul_conductivity=0.04, _insul_refl=True, _insul_quality=None, _daily_period=24.0):
        # type: (LineSegment3D, float, float, float, bool, None, float) -> None
        super(PhPipeSegment, self).__init__()
        self.geometry = _geom
        self.diameter = _diameter
        self.insulation_thickness = _insul_thickness
        self.insulation_conductivity = _insul_conductivity
        self.insulation_reflective = _insul_refl
        self.insulation_quality = _insul_quality
        self.daily_period = _daily_period

    @property
    def length(self):
        # type: () -> float
        return self.geometry.length

    def __copy__(self):
        # type: () -> PhPipeSegment
        new_obj = PhPipeSegment(self.geometry)

        new_obj.diameter = self.diameter
        new_obj.insulation_thickness = self.insulation_thickness
        new_obj.insulation_conductivity = self.insulation_conductivity
        new_obj.insulation_reflective = self.insulation_reflective
        new_obj.insulation_quality = self.insulation_quality
        new_obj.daily_period = self.daily_period
        new_obj.identifier = self.identifier
        new_obj.display_name = self.display_name
        new_obj.user_data = self.user_data

        return self

    def duplicate(self):
        # type: () -> PhPipeSegment
        return self.__copy__()

    def to_dict(self):
        # type: () -> dict[str, Union[str, dict]]
        d = {}

        d['geometry'] = self.geometry.to_dict()
        d['diameter'] = self.diameter
        d['insulation_thickness'] = self.insulation_thickness
        d['insulation_conductivity'] = self.insulation_conductivity
        d['insulation_reflective'] = self.insulation_reflective
        d['insulation_quality'] = self.insulation_quality
        d['daily_period'] = self.daily_period
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['user_data'] = self.user_data

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhPipeSegment
        new_obj = cls(
            _geom=LineSegment3D.from_dict(_input_dict['geometry'])
        )
        new_obj.diameter = _input_dict['diameter']
        new_obj.insulation_thickness = _input_dict['insulation_thickness']
        new_obj.insulation_conductivity = _input_dict['insulation_conductivity']
        new_obj.insulation_reflective = _input_dict['insulation_reflective']
        new_obj.insulation_quality = _input_dict['insulation_quality']
        new_obj.daily_period = _input_dict['daily_period']
        new_obj.identifier = _input_dict['identifier']
        new_obj.display_name = _input_dict['display_name']
        new_obj.user_data = _input_dict['user_data']

        return new_obj

    def __str__(self):
        return "{}: diam={}, length={:.3f}".format(self.__class__.__name__, self.diameter, self.length)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return self.__repr__()


class PhPipeElement(_base._PhHVACBase):
    """A Pipe Element made up of one or more individual Pipe Segments."""

    def __init__(self):
        super(PhPipeElement, self).__init__()
        self._segments = {}  # type: Dict[str, PhPipeSegment]

    @property
    def segments(self):
        # type: () -> List[PhPipeSegment]
        return list(self._segments.values())

    @property
    def length(self):
        # type: () -> float
        return sum(s.length for s in self.segments)

    def add_segment(self, _segment):
        # type: (PhPipeSegment) -> None
        self._segments[_segment.identifier] = _segment

    def __copy__(self):
        # type: () -> PhPipeElement
        new_obj = PhPipeElement()

        for segment in self.segments:
            new_obj.add_segment(segment.duplicate())

        new_obj.identifier = self.identifier
        new_obj.display_name = self.display_name
        new_obj.user_data = self.user_data

        return new_obj

    def duplicate(self):
        # type: () -> PhPipeElement
        return self.__copy__()

    def to_dict(self):
        # type: () -> dict[str, Union[str, dict]]
        d = {}

        d['segments'] = {}
        for segment in self.segments:
            d['segments'][segment.identifier] = segment.to_dict()
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['user_data'] = self.user_data

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhPipeElement
        new_obj = cls()

        for seg_dict in _input_dict['segments'].values():
            new_obj._segments[seg_dict['identifier']] = PhPipeSegment.from_dict(seg_dict)
        new_obj.identifier = _input_dict['identifier']
        new_obj.display_name = _input_dict['display_name']
        new_obj.user_data = _input_dict['user_data']

        return new_obj

    def __str__(self):
        return "{}: (display_name={}, identifier={} ) [{} segments, len={:.3f}]".format(self.__class__.__name__, self.display_name, self.identifier, len(self.segments), self.length)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return self.__repr__()


# -- Tank ---------------------------------------------------------------------


class PhSHWTankType(enumerables.CustomEnum):
    allowed = [
        "0-No storage tank",
        "1-DHW and heating",
        "2-DHW only"
    ]

    def __init__(self, _value=1, _index_offset=0):
        # type: (Union[str, int], int) -> None
        super(PhSHWTankType, self).__init__(_value, _index_offset)


class PhSHWTank(_base._PhHVACBase):
    def __init__(self):
        super(PhSHWTank, self).__init__()
        self.display_name = '_unnamed_hw_tank_'  # type: str
        self.quantity = 1  # type: int
        self._tank_type = PhSHWTankType("2-DHW only")  # type: enumerables.CustomEnum
        self.in_conditioned_space = True  # type: bool
        self.solar_connection = False  # type: bool
        self.solar_losses = 0.0  # type: float
        self.storage_loss_rate = 0.0  # type: float
        self.storage_capacity = 300  # type: float
        self.standby_losses = 4.0  # type: float
        self.standby_fraction = 0.30  # type: float
        self.room_temp = 20.0  # type: float
        self.water_temp = 60.0  # type: float

    @property
    def tank_type(self):
        # type () -> str
        return self._tank_type.value

    @tank_type.setter
    def tank_type(self, _in):
        # type: (Union[str, int]) -> None
        self._tank_type = PhSHWTankType(_in)

    def __copy__(self):
        # type: () -> PhSHWTank
        new_obj = PhSHWTank()

        new_obj.display_name = self.display_name
        new_obj.identifier = self.identifier
        new_obj.user_data = self.user_data
        new_obj.quantity = self.quantity
        new_obj._tank_type = self._tank_type
        new_obj.in_conditioned_space = self.in_conditioned_space
        new_obj.solar_connection = self.solar_connection
        new_obj.solar_losses = self.solar_losses
        new_obj.storage_loss_rate = self.storage_loss_rate
        new_obj.storage_capacity = self.storage_capacity
        new_obj.standby_losses = self.standby_losses
        new_obj.standby_fraction = self.standby_fraction
        new_obj.room_temp = self.room_temp
        new_obj.water_temp = self.water_temp

        return new_obj

    def duplicate(self):
        # type: () -> PhSHWTank
        return self.__copy__()

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = {}

        d['display_name'] = self.display_name
        d['identifier'] = self.identifier
        d['user_data'] = self.user_data
        d['quantity'] = self.quantity
        d['_tank_type'] = self._tank_type.to_dict()
        d['in_conditioned_space'] = self.in_conditioned_space
        d['solar_connection'] = self.solar_connection
        d['solar_losses'] = self.solar_losses
        d['storage_loss_rate'] = self.storage_loss_rate
        d['storage_capacity'] = self.storage_capacity
        d['standby_losses'] = self.standby_losses
        d['standby_fraction'] = self.standby_fraction
        d['room_temp'] = self.room_temp
        d['water_temp'] = self.water_temp

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhSHWTank
        obj = cls()

        obj.display_name = _input_dict['display_name']
        obj.identifier = _input_dict['identifier']
        obj.user_data = _input_dict['user_data']
        obj.quantity = _input_dict['quantity']
        obj._tank_type = PhSHWTankType.from_dict(_input_dict['_tank_type'])
        obj.in_conditioned_space = _input_dict['in_conditioned_space']
        obj.solar_connection = _input_dict['solar_connection']
        obj.solar_losses = _input_dict['solar_losses']
        obj.storage_loss_rate = _input_dict['storage_loss_rate']
        obj.storage_capacity = _input_dict['storage_capacity']
        obj.standby_losses = _input_dict['standby_losses']
        obj.standby_fraction = _input_dict['standby_fraction']
        obj.room_temp = _input_dict['room_temp']
        obj.water_temp = _input_dict['water_temp']

        return obj

    def __str__(self):
        return "{}: {} {}".format(self.__class__.__name__, self.tank_type, self.display_name)

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.display_name)

    def ToString(self):
        return self.__repr__()


# -- Heaters ------------------------------------------------------------------


class PhHotWaterHeater(_base._PhHVACBase):
    """Base class for all PH Hot-Water Heaters."""

    def __init__(self):
        super(PhHotWaterHeater, self).__init__()
        self.percent_coverage = 1.0
        self.in_conditioned_space = True

    def to_dict(self):
        # type: () -> dict
        d = {}
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHotWaterHeater
        new_obj = cls()
        return new_obj

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)


class PhSHWHeaterElectric(PhHotWaterHeater):
    def __init__(self):
        super(PhSHWHeaterElectric, self).__init__()

    def to_dict(self):
        # type: () -> dict
        d = {}

        d['heater_type'] = self.__class__.__name__
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['percent_coverage'] = self.percent_coverage
        d['in_conditioned_space'] = self.in_conditioned_space

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhSHWHeaterElectric
        new_obj = cls()

        new_obj.identifier = _input_dict['identifier']
        new_obj.display_name = _input_dict['display_name']
        new_obj.percent_coverage = _input_dict['percent_coverage']
        new_obj.in_conditioned_space = _input_dict['in_conditioned_space']

        return new_obj


class PhSHWHeaterBoiler(PhHotWaterHeater):
    def __init__(self):
        super(PhSHWHeaterBoiler, self).__init__()
        self.fuel = 1  # Gas
        self.condensing = True
        self.effic_at_30_perc_load = 0.98
        self.effic_at_nominal_load = 0.94
        self.avg_return_temp_at_30_perc_load = 30
        self.avg_boiler_temp_at_70_55 = 41
        self.avg_boiler_temp_at_55_45 = 35
        self.avg_boiler_temp_at_35_28 = 24

    def to_dict(self):
        # type: () -> dict
        d = {}

        d['heater_type'] = self.__class__.__name__
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['percent_coverage'] = self.percent_coverage
        d['in_conditioned_space'] = self.in_conditioned_space

        d['fuel'] = self.fuel
        d['condensing'] = self.condensing
        d['effic_at_30_perc_load'] = self.effic_at_30_perc_load
        d['effic_at_nominal_load'] = self.effic_at_nominal_load
        d['avg_return_temp_at_30_perc_load'] = self.avg_return_temp_at_30_perc_load
        d['avg_boiler_temp_at_70_55'] = self.avg_boiler_temp_at_70_55
        d['avg_boiler_temp_at_55_45'] = self.avg_boiler_temp_at_55_45
        d['avg_boiler_temp_at_35_28'] = self.avg_boiler_temp_at_35_28

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhSHWHeaterBoiler
        new_obj = cls()

        new_obj.identifier = _input_dict['identifier']
        new_obj.display_name = _input_dict['display_name']
        new_obj.percent_coverage = _input_dict['percent_coverage']
        new_obj.in_conditioned_space = _input_dict['in_conditioned_space']

        new_obj.fuel = _input_dict['fuel']
        new_obj.condensing = _input_dict['condensing']
        new_obj.effic_at_30_perc_load = _input_dict['effic_at_30_perc_load']
        new_obj.effic_at_nominal_load = _input_dict['effic_at_nominal_load']
        new_obj.avg_return_temp_at_30_perc_load = _input_dict['avg_return_temp_at_30_perc_load']
        new_obj.avg_boiler_temp_at_70_55 = _input_dict['avg_boiler_temp_at_70_55']
        new_obj.avg_boiler_temp_at_55_45 = _input_dict['avg_boiler_temp_at_55_45']
        new_obj.avg_boiler_temp_at_35_28 = _input_dict['avg_boiler_temp_at_35_28']

        return new_obj


class PhSHWHeaterBoilerWood(PhHotWaterHeater):
    def __init__(self):
        super(PhSHWHeaterBoilerWood, self).__init__()
        self.fuel = 1  # Pellet
        self.effic_in_basic_cycle = 0.72
        self.effic_in_const_operation = 0.80
        self.avg_frac_heat_released = 0.5
        self.on_off_temp_diff = 30

    def to_dict(self):
        # type: () -> dict
        d = {}

        d['heater_type'] = self.__class__.__name__
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['percent_coverage'] = self.percent_coverage
        d['in_conditioned_space'] = self.in_conditioned_space

        d['fuel'] = self.fuel
        d['effic_in_basic_cycle'] = self.effic_in_basic_cycle
        d['effic_in_const_operation'] = self.effic_in_const_operation
        d['avg_frac_heat_released'] = self.avg_frac_heat_released
        d['on_off_temp_diff'] = self.on_off_temp_diff

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        new_obj = cls()

        new_obj.identifier = _input_dict['identifier']
        new_obj.display_name = _input_dict['display_name']
        new_obj.percent_coverage = _input_dict['percent_coverage']
        new_obj.in_conditioned_space = _input_dict['in_conditioned_space']

        new_obj.fuel = _input_dict['fuel']
        new_obj.effic_in_basic_cycle = _input_dict['effic_in_basic_cycle']
        new_obj.effic_in_const_operation = _input_dict['effic_in_const_operation']
        new_obj.avg_frac_heat_released = _input_dict['avg_frac_heat_released']
        new_obj.on_off_temp_diff = _input_dict['on_off_temp_diff']

        return new_obj


class PhSHWHeaterDistrict(PhHotWaterHeater):
    def __init__(self):
        super(PhSHWHeaterDistrict, self).__init__()
        self.energy_carrier = 1
        self.solar_fraction = 0
        self.util_fact_heat_transfer = 1

    def to_dict(self):
        # type: () -> dict
        d = {}

        d['heater_type'] = self.__class__.__name__
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['percent_coverage'] = self.percent_coverage
        d['in_conditioned_space'] = self.in_conditioned_space

        d['energy_carrier'] = self.energy_carrier
        d['solar_fraction'] = self.solar_fraction
        d['util_fact_heat_transfer'] = self.util_fact_heat_transfer

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        new_obj = cls()

        new_obj.identifier = _input_dict['identifier']
        new_obj.display_name = _input_dict['display_name']
        new_obj.percent_coverage = _input_dict['percent_coverage']
        new_obj.in_conditioned_space = _input_dict['in_conditioned_space']

        new_obj.energy_carrier = _input_dict['energy_carrier']
        new_obj.solar_fraction = _input_dict['solar_fraction']
        new_obj.util_fact_heat_transfer = _input_dict['util_fact_heat_transfer']

        return new_obj


class PhSHWHeaterHeatPump(PhHotWaterHeater):
    def __init__(self):
        super(PhSHWHeaterHeatPump, self).__init__()
        self.annual_COP = None
        self.annual_system_perf_ratio = None
        self.annual_energy_factor = None

    def to_dict(self):
        # type: () -> dict
        d = {}

        d['heater_type'] = self.__class__.__name__
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['percent_coverage'] = self.percent_coverage
        d['in_conditioned_space'] = self.in_conditioned_space

        d['annual_COP'] = self.annual_COP
        d['annual_system_perf_ratio'] = self.annual_system_perf_ratio
        d['annual_energy_factor'] = self.annual_energy_factor

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        new_obj = cls()

        new_obj.identifier = _input_dict['identifier']
        new_obj.display_name = _input_dict['display_name']
        new_obj.percent_coverage = _input_dict['percent_coverage']
        new_obj.in_conditioned_space = _input_dict['in_conditioned_space']

        new_obj.annual_COP = _input_dict['annual_COP']
        new_obj.annual_system_perf_ratio = _input_dict['annual_system_perf_ratio']
        new_obj.annual_energy_factor = _input_dict['annual_energy_factor']

        return new_obj


# -----------------------------------------------------------------------------


class PhSHWHeaterBuilder(object):
    """Constructor class for Hot-Water-Heater objects"""

    heaters = {
        'PhSHWHeaterElectric': PhSHWHeaterElectric,
        'PhSHWHeaterBoiler': PhSHWHeaterBoiler,
        'PhSHWHeaterBoilerWood': PhSHWHeaterBoilerWood,
        'PhSHWHeaterDistrict': PhSHWHeaterDistrict,
        'PhSHWHeaterHeatPump': PhSHWHeaterHeatPump,
    }

    @ classmethod
    def from_dict(cls, _input_dict):
        # type: (dict) -> PhHotWaterHeater

        heater_type = _input_dict.get('heater_type')
        if heater_type is None:
            raise UnknownPhHeaterTypeError(cls.heaters.keys(), heater_type)

        heater_class = cls.heaters[heater_type]
        new_heater = heater_class.from_dict(_input_dict)

        return new_heater

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- Python Version: 2.7 -*-
# -*- coding: utf-8 -*-

"""Helper/Temp Classes for calculating Phius Multifamily Elec. Energy"""

try:
    from typing import ValuesView
except ImportError:
    pass  # IronPython 2.7

from honeybee import room
from honeybee_energy.schedule import ruleset
from honeybee_energy.load import equipment
from honeybee_ph import space


class PhiusResidentialStory(object):
    """Represents one Residential Story of the Phius Multifamily Calculator."""

    def __init__(self, hb_room_list):
        self.lighting_int_HE_frac = 1.0
        self.lighting_ext_HE_frac = 1.0
        self.lighting_garage_HE_frac = 1.0

        self.story_number = hb_room_list[0].story
        self.total_floor_area_m2 = self.calc_story_floor_area(hb_room_list)
        self.total_number_dwellings = len(hb_room_list)
        self.total_number_bedrooms = self.calc_story_bedrooms(hb_room_list)

        self.design_occupancy = self.calc_design_occupancy()
        self.mel = self.calc_mel()
        self.lighting_int = self.calc_lighting_int()
        self.lighting_ext = self.calc_lighting_ext()
        self.lighting_garage = self.calc_lighting_garage()

    @property
    def total_floor_area_ft2(self):
        # type () -> float
        return self.total_floor_area_m2 * 10.7639

    def calc_story_floor_area(self, _hb_rooms):
        # type: (list[room.Room]) -> float
        story_floor_area = 0
        for rm in _hb_rooms:
            for space in rm.properties.ph.spaces:
                story_floor_area += space.weighted_floor_area
        return story_floor_area

    def calc_story_bedrooms(self, _hb_rooms):
        # type: (list[room.Room]) -> int
        return sum(rm.properties.energy.people.properties.ph.number_bedrooms for rm in _hb_rooms)

    def calc_design_occupancy(self):
        # type: () -> float
        return ((self.total_number_bedrooms / self.total_number_dwellings) + 1) * self.total_number_dwellings

    def calc_mel(self):
        # type: () -> float
        return (self.total_number_dwellings * 413 + 69 * self.total_number_bedrooms + 0.91 * self.total_floor_area_ft2) * 0.8

    def calc_lighting_int(self):
        # type: () -> float
        return (0.2+0.8*(4-3*self.lighting_int_HE_frac)/3.7)*(self.total_number_dwellings*455+0.8*self.total_floor_area_ft2)*0.8

    def calc_lighting_ext(self):
        # type: () -> float
        return (1-0.75*self.lighting_ext_HE_frac)*(self.total_number_dwellings*100+0.05*self.total_floor_area_ft2)*0.8

    def calc_lighting_garage(self):
        # type: () -> float
        return self.total_number_dwellings*(100*(1-self.lighting_garage_HE_frac)+25*self.lighting_garage_HE_frac)*0.8

    def __str__(self):
        return "{}(total_floor_area_m2={}, total_number_dwellings={}, total_number_bedrooms={})".format(
            self.__class__.__name__, self.total_floor_area_m2, self.total_number_dwellings, self.total_number_bedrooms)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)

# -----------------------------------------------------------------------------
# -- Non-Res


class PhiusNonResProgram(object):
    """An individual Phius Non-Res Program Type."""

    def __init__(self):
        self.name = "__unnamed_nonres_program_"
        self.usage_days_yr = 365
        self.operating_hours_day = 0
        self.lighting_W_per_m2 = 0
        self.mel_kWh_m2_yr = 0

    @property
    def lighting_W_per_ft2(self):
        # type: () -> float
        return self.lighting_W_per_m2 * 0.09290304

    @property
    def mel_kWh_ft2_yr(self):
        return self.mel_kWh_m2_yr * 0.09290304

    def to_phius_mf_workbook(self):
        # type: () -> str
        """Returns a text block formated to match the Phius MF Calculator."""
        return ",".join([
            str(self.name),
            str(self.usage_days_yr),
            str(self.operating_hours_day),
            str(self.lighting_W_per_ft2),
            str(self.mel_kWh_ft2_yr),
        ])

    @classmethod
    def from_hb_room(cls, _hb_room):
        # type: (room.Room) -> PhiusNonResProgram
        """Returns a new PhiusNonResProgram object with attributes based on an HBE-Room."""
        obj = cls()

        obj.name = _hb_room.properties.energy.program_type.display_name
        obj.operating_hours_day = obj._operating_hours_day_from_hb_schedule(
            _hb_room.properties.energy.lighting.schedule)
        obj.lighting_W_per_m2 = _hb_room.properties.energy.lighting.watts_per_area
        obj.mel_kWh_m2_yr = obj._calc_mel_kWh_per_m2_from_hb_elec(
            _hb_room.properties.energy.electric_equipment
        )

        return obj

    def _calc_mel_kWh_per_m2_from_hb_elec(self, _hb_elec_equip):
        # type: (equipment.ElectricEquipment) -> float
        """Returns the total Elec. Equip kWh for the HBE-Electric-Equipment."""

        # -- Calc annual full-load hours
        annual_full_load_hours = sum(_hb_elec_equip.schedule.values())

        # -- Calc total usage in kWh
        return (_hb_elec_equip.watts_per_area * annual_full_load_hours) / 1000

    def _operating_hours_day_from_hb_schedule(self, _hb_lght_sched):
        # type: (ruleset.ScheduleRuleset) -> float
        """Return a daily operating period (num. hours) from an HB-Lighting-Schedule"""
        operating_frac = sum(_hb_lght_sched.values()) / len(_hb_lght_sched.values())
        return operating_frac * 24

    def __str__(self):
        return '{}(name={})'.format(self.__class__.__name__, self.name)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)


class PhiusNonResProgramCollection(object):
    """Collection of Phius Non-Res Program Types."""

    def __init__(self):
        self._collection = {}

    def add_program(self, _program, _key=None):
        # type: (PhiusNonResProgram, str | None) -> None
        """Adds a new PhiusNonResProgram to the collection."""
        if not _key:
            _key = _program.name
        self._collection[_key] = _program

    @property
    def programs(self):
        # type: () -> ValuesView[PhiusNonResProgram]
        """Returns a list of the PhiusNonResPrograms in the collection."""
        return self._collection.values()

    def __getitem__(self, key):
        # type: (str) -> PhiusNonResProgram
        return self._collection[key]

    def to_phius_mf_workbook(self):
        # type: () -> lisr
        """Returns a text block formated to match the Phius MF Calculator."""
        return [prog.to_phius_mf_workbook() for prog in self.programs]


class PhiusNonResRoom(object):
    """A single Phius Non-Res Space."""

    def __init__(self):
        self.multiplier = 1
        self.occupancy_sensor = "N"
        self.name = "_unnamed_phius_nonres_room"
        self.icfa_m2 = 0.0
        self.misc_mel = 0
        self.program_type = PhiusNonResProgram()

    @property
    def icfa_ft2(self):
        # type: () -> float
        return self.icfa_m2 * 10.7639

    @property
    def mel_kWh_yr(self):
        # type: () -> float
        """Total yearly MEL kWh EXCLUDING the Misc MEL"""
        return (self.icfa_m2 * self.program_type.mel_kWh_m2_yr)

    @property
    def total_mel_kWh(self):
        """Total yearly MEL kWh INCLUDING the Misc MEL"""
        # type: () -> float
        return self.mel_kWh_yr + self.misc_mel

    @property
    def total_lighting_kWh(self):
        # type: () -> float
        return (self.program_type.usage_days_yr * self.program_type.operating_hours_day *
                self.program_type.lighting_W_per_m2 * self.icfa_m2) / 1000

    def to_phius_mf_workbook(self):
        # type: () -> str
        """Returns a string representation that matches the Phius MF Calculator."""
        return ",".join([
            str(self.multiplier),
            str(self.occupancy_sensor),
            str(self.name),
            str(self.program_type.name),
            str(self.misc_mel),
            str(self.icfa_ft2),
        ])

    def to_phius_mf_workbook_results(self):
        return ",".join([
            str(self.program_type.lighting_W_per_ft2),
            str(self.program_type.usage_days_yr),
            str(self.program_type.operating_hours_day),
            str(self.program_type.mel_kWh_ft2_yr),
            str(self.total_lighting_kWh),
            str(self.mel_kWh_yr),
        ])

    @classmethod
    def from_ph_space(cls, _ph_space):
        # type: (space.Space) -> PhiusNonResRoom
        """Returns a new PhiusNonResSpace with properties based on a PH-Space."""
        obj = cls()

        obj.name = _ph_space.full_name
        obj.icfa_m2 = _ph_space.weighted_floor_area
        obj.program_type = PhiusNonResProgram.from_hb_room(_ph_space.host)

        return obj

    def __str__(self):
        return '{}(name={}, program_type={},'\
            'misc_mel={}, icfa={})'.format(
                self.__class__.__name__, self.name, self.program_type, self.misc_mel, self.icfa_m2)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Passive House HVAC Equipment Classes"""

try:
    from typing import Any, Optional
except ImportError:
    pass  # IronPython

from honeybee_energy_ph.hvac import _base


class Ventilator(_base._PhHVACBase):
    def __init__(self):
        super(Ventilator, self).__init__()
        self.display_name = '_unnamed_ventilator_'
        self.id_num = 0
        self.quantity = 1
        self.sensible_heat_recovery = 0.0
        self.latent_heat_recovery = 0.0
        self.electric_efficiency = 0.55
        self.frost_protection_reqd = True
        self.temperature_below_defrost_used = -5
        self.in_conditioned_space = True

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = {}

        d['display_name'] = self.display_name
        d['quantity'] = self.quantity
        d['sensible_heat_recovery'] = self.sensible_heat_recovery
        d['latent_heat_recovery'] = self.latent_heat_recovery
        d['electric_efficiency'] = self.electric_efficiency
        d['frost_protection_reqd'] = self.frost_protection_reqd
        d['temperature_below_defrost_used'] = self.temperature_below_defrost_used
        d['in_conditioned_space'] = self.in_conditioned_space

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> Ventilator
        obj = cls()

        obj.display_name = _input_dict['display_name']
        obj.quantity = _input_dict['quantity']
        obj.sensible_heat_recovery = _input_dict['sensible_heat_recovery']
        obj.latent_heat_recovery = _input_dict['latent_heat_recovery']
        obj.electric_efficiency = _input_dict['electric_efficiency']
        obj.frost_protection_reqd = _input_dict['frost_protection_reqd']
        obj.temperature_below_defrost_used = _input_dict['temperature_below_defrost_used']
        obj.in_conditioned_space = _input_dict['in_conditioned_space']

        return obj

    def __repr__(self):
        return "{}(display_name={!r}, sensible_heat_recovery={:0.2f})".format(self.__class__.__name__, self.display_name, self.sensible_heat_recovery)

    def ToString(self):
        return self.__repr__()


class PhVentilationSystem(_base._PhHVACBase):
    """Passive House Fresh-Air Ventilation System."""

    def __init__(self):
        super(PhVentilationSystem, self).__init__()
        self.display_name = '_unnamed_ph_vent_system_'
        self.sys_type = 1  # '1-Balanced PH ventilation with HR'
        self.duct_01 = None
        self.duct_02 = None
        self._ventilation_unit = None  # type: Optional[Ventilator]

    @property
    def ventilation_unit(self):
        # type: () -> Optional[Ventilator]
        return self._ventilation_unit

    @ventilation_unit.setter
    def ventilation_unit(self, _in):
        # type: (Optional[Ventilator]) -> None
        self._ventilation_unit = _in

        if not self._ventilation_unit:
            return None

        if self._ventilation_unit.display_name == '_unnamed_ventilator_':
            self._ventilation_unit.display_name = self.display_name
        return None

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = {}

        d['identifier'] = str(self.identifier)
        d['name'] = self.display_name
        d['sys_type'] = self.sys_type
        d['duct_01'] = self.duct_01
        d['duct_02'] = self.duct_02

        if self.ventilation_unit:
            d['ventilation_unit'] = self.ventilation_unit.to_dict()

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhVentilationSystem
        obj = cls()

        obj.identifier = _input_dict['identifier']
        obj.display_name = _input_dict['name']
        obj.sys_type = _input_dict['sys_type']
        obj.duct_01 = _input_dict['duct_01']
        obj.duct_02 = _input_dict['duct_02']

        vent_unit_dict = _input_dict.get('ventilation_unit', None)
        if vent_unit_dict:
            obj.ventilation_unit = Ventilator.from_dict(vent_unit_dict)

        return obj

    def __repr__(self):
        return "{}(display_name={!r}, sys_type={!r})".format(self.__class__.__name__, self.display_name, self.sys_type)

    def ToString(self):
        return self.__repr__()



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""HBPH Cooling Objects"""

try:
    from typing import Any
except ImportError:
    pass  # IronPython 2.7
import sys
from honeybee_energy_ph.hvac import _base

# -----------------------------------------------------------------------------
# Cooling Base


class UnknownPhCoolingTypeError(Exception):
    def __init__(self, _heater_types, _received_type):
        # type: (list[str], str) -> None
        self.msg = 'Error: Unknown HBPH-Cooling-SubSystem type? Got: "{}" but only types: {} are allowed?'.format(
            _received_type, _heater_types)
        super(UnknownPhCoolingTypeError, self).__init__(self.msg)


class PhCoolingSystem(_base._PhHVACBase):
    """Base class for all HBPH-Cooling-Systems."""

    def __init__(self):
        super(PhCoolingSystem, self).__init__()
        self.cooling_class_name = self.__class__.__name__
        self.percent_coverage = 1.0

    def to_dict(self):
        # type: () -> dict
        d = {}
        d['identifier'] = self.identifier
        d['display_name'] = self.display_name
        d['cooling_class_name'] = self.cooling_class_name
        d['percent_coverage'] = self.percent_coverage
        return d

    def base_attrs_from_dict(self, _input_dict):
        # type: (PhCoolingSystem, dict) -> PhCoolingSystem
        self.identifier = _input_dict['identifier']
        self.display_name = _input_dict['display_name']
        self.cooling_class_name = _input_dict['cooling_class_name']
        self.percent_coverage = _input_dict['percent_coverage']
        return self

    def check_dict_type(self, _input_dict):
        # type: (dict) -> None
        """Check that the input dict type is correct for the Heating System being constructed."""
        cooling_class_name = _input_dict['cooling_class_name']
        msg = "Error creating Cooling System from dict. Expected '{}' but got '{}'".format(
            self.__class__.__name__, cooling_class_name)
        assert cooling_class_name == str(self.__class__.__name__), msg
        return None


# -----------------------------------------------------------------------------
# Cooling Types


class PhCoolingVentilation(PhCoolingSystem):
    """Cooling via the Fresh-Air Ventilation System (ERV)."""

    def __init__(self):
        super(PhCoolingVentilation, self).__init__()
        self.single_speed = False
        self.min_coil_temp = 12
        self.capacity = 10
        self.annual_COP = 4

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhCoolingVentilation, self).to_dict()
        d['single_speed'] = self.single_speed
        d['min_coil_temp'] = self.min_coil_temp
        d['capacity'] = self.capacity
        d['annual_COP'] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhCoolingVentilation
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.single_speed = _input_dict['single_speed']
        new_obj.min_coil_temp = _input_dict['min_coil_temp']
        new_obj.capacity = _input_dict['capacity']
        new_obj.annual_COP = _input_dict['annual_COP']
        return new_obj


class PhCoolingRecirculation(PhCoolingSystem):
    """Cooling via a 'recirculation' system (typical AC)."""

    def __init__(self):
        super(PhCoolingRecirculation, self).__init__()
        self.single_speed = False
        self.min_coil_temp = 12
        self.flow_rate_m3_hr = 100
        self.flow_rate_variable = True
        self.capacity = 10
        self.annual_COP = 4

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhCoolingRecirculation, self).to_dict()
        d['single_speed'] = self.single_speed
        d['min_coil_temp'] = self.min_coil_temp
        d['flow_rate_m3_hr'] = self.flow_rate_m3_hr
        d['flow_rate_variable'] = self.flow_rate_variable
        d['capacity'] = self.capacity
        d['annual_COP'] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhCoolingRecirculation
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.single_speed = _input_dict['single_speed']
        new_obj.min_coil_temp = _input_dict['min_coil_temp']
        new_obj.flow_rate_m3_hr = _input_dict['flow_rate_m3_hr']
        new_obj.flow_rate_variable = _input_dict['flow_rate_variable']
        new_obj.capacity = _input_dict['capacity']
        new_obj.annual_COP = _input_dict['annual_COP']
        return new_obj


class PhCoolingDehumidification(PhCoolingSystem):
    """Cooling via dedicated dehumidification system."""

    def __init__(self):
        super(PhCoolingDehumidification, self).__init__()
        self.useful_heat_loss = False
        self.annual_COP = 4

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhCoolingDehumidification, self).to_dict()
        d['useful_heat_loss'] = self.useful_heat_loss
        d['annual_COP'] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhCoolingDehumidification
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.useful_heat_loss = _input_dict['useful_heat_loss']
        new_obj.annual_COP = _input_dict['annual_COP']
        return new_obj


class PhCoolingPanel(PhCoolingSystem):
    """Cooling via radiant panels."""

    def __init__(self):
        super(PhCoolingPanel, self).__init__()
        self.annual_COP = 4

    def to_dict(self):
        # type: () -> dict[str, Any]
        d = super(PhCoolingPanel, self).to_dict()
        d['annual_COP'] = self.annual_COP
        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhCoolingPanel
        new_obj = cls()
        new_obj.check_dict_type(_input_dict)
        new_obj.base_attrs_from_dict(_input_dict)
        new_obj.annual_COP = _input_dict['annual_COP']
        return new_obj


# -----------------------------------------------------------------------------


class PhCoolingSystemBuilder(object):
    """Constructor class for HBPH-CoolingSystems"""

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (dict[str, Any]) -> PhCoolingSystem
        """Find the right cooling constructor class from the module based on the 'type' name."""
        cooling_class_name = _input_dict.get('cooling_class_name')

        valid_class_types = [nm for nm in dir(
            sys.modules[__name__]) if nm.startswith('Ph')]

        if cooling_class_name not in valid_class_types:
            raise UnknownPhCoolingTypeError(valid_class_types, cooling_class_name)
        cooling_class = getattr(sys.modules[__name__], cooling_class_name)
        new_equipment = cooling_class.from_dict(_input_dict)
        return new_equipment

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""PH 'Space' and Related Sub-object Classes (FloorSegments, etc)"""

try:
    from typing import Any, Optional, List, Dict
except:
    pass  # IronPython

from honeybee_ph import _base
from honeybee_ph.properties.space import SpaceProperties
from ladybug_geometry import geometry3d


class SpaceFloorSegment(_base._Base):
    def __init__(self):
        super(SpaceFloorSegment, self).__init__()
        self.geometry = None  # type: Optional[geometry3d.face.Face3D]
        self.weighting_factor = 1.0

        # -- Reference Point Note: Usually this is just the center, although for
        # -- more complex shaped like 'L' and 'U' it cannot just be the center.
        # -- In those cases, it should be a point 'on' the surface, as near to the
        # -- center as possible. This point is used for testing hosting of the
        # -- SpaceFloorSegment 'inside' an HB-Room.
        self.reference_point = None  # type: Optional[geometry3d.pointvector.Point3D]

    @property
    def weighted_floor_area(self):
        # type: () -> float
        """The floor area of the floor segment weighted by any reduction factors (iFCA, TFA)"""
        if self.geometry:
            return self.geometry.area * self.weighting_factor
        else:
            return 0

    @property
    def floor_area(self):
        # type: () -> float
        """The floor area of the floor segment UN-weighted by any reduction factors (iFCA, TFA)"""
        if self.geometry:
            return self.geometry.area
        else:
            return 0

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = {}

        d['weighting_factor'] = self.weighting_factor
        if self.geometry:
            d['geometry'] = self.geometry.to_dict()
        if self.reference_point:
            d['reference_point'] = self.reference_point.to_dict()

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> SpaceFloorSegment
        new_obj = cls()

        new_obj.weighting_factor = _input_dict.get('weighting_factor', 1.0)

        geom_dict = _input_dict.get('geometry', None)
        if geom_dict:
            new_obj.geometry = geometry3d.Face3D.from_dict(geom_dict)

        ref_pt_dict = _input_dict.get('reference_point', None)
        if ref_pt_dict:
            new_obj.reference_point = geometry3d.Point3D.from_dict(ref_pt_dict)

        return new_obj

    def duplicate(self):
        # type: () -> SpaceFloorSegment
        new_obj = SpaceFloorSegment()

        if self.geometry:
            new_obj.geometry = self.geometry.duplicate()

        if self.reference_point:
            new_obj.reference_point = self.reference_point.duplicate()

        new_obj.weighting_factor = self.weighting_factor

        return new_obj

    def __str__(self):
        return '{}(weighting_factor={!r}, geometry={!r}, reference_point={!r})'.format(self.__class__.__name__,
                                                                                       self.weighting_factor,
                                                                                       self.geometry,
                                                                                       self.reference_point)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)


class SpaceFloor(_base._Base):
    def __init__(self):
        super(SpaceFloor, self).__init__()
        self._floor_segments = list()  # list[SpaceFloorSegment]
        self.geometry = None  # geometry3d.Face3D

    @property
    def reference_points(self):
        # type() -> list[geometry3d.Point3D]
        """Returns a list of the Floor's FloorSegment reference points."""
        return [seg.reference_point for seg in self.floor_segments]

    @property
    def weighted_floor_area(self):
        # type: () ->  float
        """The total floor area of all floor segments, weighted by any reduction factors (iFCA, TFA)"""
        return sum((seg.weighted_floor_area for seg in self.floor_segments))

    @property
    def floor_area(self):
        # type: () ->  float
        """The total floor area of all floor segments, UN-weighted by any reduction factors (iFCA, TFA)"""
        return sum((seg.floor_area for seg in self.floor_segments))

    def add_floor_segment(self, _floor_seg):
        # type: (SpaceFloorSegment) -> None
        """Add a new SpaceFloorSegment to the SpaceFloor.

        Arguments:
        ----------
            * _floor_seg (SpaceFloorSegment): The SpaceFloorSegment to add to the SpaceFloor.

        Returns:
        --------
            * None
        """
        if not _floor_seg:
            return
        self._floor_segments.append(_floor_seg)

    @property
    def floor_segments(self):
        # type: () -> list[SpaceFloorSegment]
        return self._floor_segments

    def duplicate(self):
        # type: () -> SpaceFloor
        new_floor = SpaceFloor()
        if self.geometry:
            new_floor.geometry = self.geometry.duplicate()
        for seg in self.floor_segments:
            new_floor.add_floor_segment(seg.duplicate())
        return new_floor

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = {}

        d['floor_segments'] = [seg.to_dict() for seg in self.floor_segments]
        d['geometry'] = self.geometry.to_dict() if self.geometry else None

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> SpaceFloor
        new_obj = cls()

        geom_dict = _input_dict.get('geometry', None)
        if geom_dict:
            new_obj.geometry = geometry3d.Face3D.from_dict(geom_dict)

        flr_seg_dicts = _input_dict.get('floor_segments', [])
        for seg_dict in flr_seg_dicts:
            new_obj.add_floor_segment(SpaceFloorSegment.from_dict(seg_dict))

        return new_obj

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)


class SpaceVolume(_base._Base):
    def __init__(self):
        super(SpaceVolume, self).__init__()
        self.avg_ceiling_height = 2.5  # m
        self.floor = SpaceFloor()
        self.geometry = list()

    @property
    def net_volume(self):
        # type: () -> float
        return self.floor_area * self.avg_ceiling_height

    @property
    def weighted_floor_area(self):
        # type: () -> float
        """The total floor area of all floor segments in the Volume, weighted by any reduction factors (iFCA, TFA)"""
        return self.floor.weighted_floor_area

    @property
    def floor_area(self):
        # type: () -> float
        """The total floor area of all floor segments in the Volume, UN-weighted by any reduction factors (iFCA, TFA)"""
        return self.floor.floor_area

    @property
    def reference_points(self):
        # type() -> list[geometry3d.Point3D]
        """Returns the Volume's FloorSegment reference points (center)."""
        return self.floor.reference_points

    @property
    def floor_segment_surfaces(self):
        # type: () -> List[Optional[geometry3d.face.Face3D]]
        return [flr_seg.geometry for flr_seg in self.floor.floor_segments]

    def duplicate(self):
        # type: () -> SpaceVolume
        new_volume = SpaceVolume()
        new_volume.avg_ceiling_height = self.avg_ceiling_height
        new_volume.floor = self.floor.duplicate()
        if self.geometry:
            new_volume.geometry = [geo.duplicate() for geo in self.geometry]
        return new_volume

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = {}

        d['avg_ceiling_height'] = self.avg_ceiling_height
        d['floor'] = self.floor.to_dict()
        d['geometry'] = [geom.to_dict() for geom in self.geometry]

        return d

    @classmethod
    def from_dict(cls, _input_dict):
        # type: (Dict[str, Any]) -> SpaceVolume
        new_obj = cls()

        new_obj.avg_ceiling_height = _input_dict.get("avg_ceiling_height")
        new_obj.floor = SpaceFloor.from_dict(_input_dict.get("floor", {}))

        geom_list = _input_dict.get("geometry", [])
        for geom_dict in geom_list:
            new_obj.geometry.append(geometry3d.Face3D.from_dict(geom_dict))

        return new_obj

    def __str__(self):
        return '{}()'.format(self.__class__.__name__)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)


class Space(_base._Base):

    def __init__(self, _host=None):
        super(Space, self).__init__()
        self.quantity = 1
        self.wufi_type = 99  # -- User-Defined
        self.name = ''
        self.number = ''
        self.host = _host

        self._volumes = list()
        self.properties = SpaceProperties(self)

    @property
    def display_name(self):
        # type: () -> str
        return "{}: {}-{}".format(self.__class__.__name__, self.number, self.name)

    @property
    def full_name(self):
        # type: () -> str
        return "{}-{}".format(self.number, self.name)

    @property
    def net_volume(self):
        # type: () -> float
        return sum([vol.net_volume for vol in self.volumes])

    @property
    def avg_clear_height(self):
        # type: () -> float
        """Returns the average floor-area-weighted height of all the Volumes in the Space"""

        total_weighted_height = 0
        for vol in self.volumes:
            total_weighted_height += vol.floor_area * vol.avg_ceiling_height

        return total_weighted_height / self.floor_area

    @property
    def weighted_floor_area(self):
        # type: () -> float
        """The total floor area of all floor segments in the Space, weighted by any reduction factors (iFCA, TFA)"""
        return sum((vol.weighted_floor_area for vol in self.volumes))

    @property
    def floor_area(self):
        # type: () -> float
        """The total floor area of all floor segments in the Space, UN-weighted by any reduction factors (iFCA, TFA)"""
        return sum((vol.floor_area for vol in self.volumes))

    @property
    def average_floor_weighting_factor(self):
        # type: () -> float
        """Returns the average weighting factor (TFA/iCFA) for the Space's floor-segments."""
        return self.weighted_floor_area / self.floor_area

    @property
    def reference_points(self):
        # type: () -> list[geometry3d.Point3D]
        """Returns a list of the Space's Volume reference Points (center of the floor segments)."""
        pts = []
        for vol in self.volumes:
            pts += vol.reference_points
        return pts

    @property
    def volumes(self):
        # type: () -> List[SpaceVolume]
        return self._volumes

    @property
    def floor_segment_surfaces(self):
        # type: () -> List[List[Optional[geometry3d.face.Face3D]]]
        return [v.floor_segment_surfaces for v in self.volumes]

    def add_new_volumes(self, _new_volumes):
        # type: (list[SpaceVolume]) -> None
        """Add a new SpaceVolume or list of SpaceVolumes to the Space.

        Arguments:
        ----------
            * _new_volumes (list[SpaceVolume]): A list of the SpaceVolumes to add.

        Returns:
        --------
            *  None
        """
        if not isinstance(_new_volumes, (set, tuple, list)):
            _new_volumes = [_new_volumes]

        for new_vol in _new_volumes:
            self._volumes.append(new_vol)

    def duplicate(self, _host=None):
        # type: (Any) -> Space
        new_space = Space()
        if _host:
            new_space.host = _host
        else:
            new_space.host = self.host

        new_space.quantity = self.quantity
        new_space.wufi_type = self.wufi_type
        new_space.name = self.name
        new_space.number = self.number
        new_space.add_new_volumes([vol.duplicate() for vol in self.volumes])
        new_space.properties._duplicate_extension_attr(self.properties)

        return new_space

    def to_dict(self):
        # type: () -> Dict[str, Any]
        d = {}

        d['quantity'] = self.quantity
        d['wufi_type'] = self.wufi_type
        d['name'] = self.name
        d['number'] = self.number
        d['volumes'] = [vol.to_dict() for vol in self.volumes]
        d['properties'] = self.properties.to_dict()

        return d

    @classmethod
    def from_dict(cls, _input_dict, _host):
        # type: (Dict[str, Any], Any) -> Space
        new_obj = cls(_host)

        new_obj.quantity = _input_dict["quantity"]
        new_obj.wufi_type = _input_dict["wufi_type"]
        new_obj.name = _input_dict["name"]
        new_obj.number = _input_dict["number"]
        new_obj.add_new_volumes(
            [SpaceVolume.from_dict(d) for d in _input_dict["volumes"]]
        )
        new_obj.properties = SpaceProperties.from_dict(
            _input_dict["properties"],
            _host=new_obj
        )
        new_obj.properties._load_extension_attr_from_dict(
            _input_dict["properties"]
        )
        return new_obj

    def __str__(self):
        return '{}(name={!r}, number={!r}, volumes={!r})'.format(self.__class__.__name__,
                                                                 self.name, self.number, self.volumes)

    def __repr__(self):
        return str(self)

    def ToString(self):
        return str(self)



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Default data for Residential PH Electrical Appliances (PHI and Phius)."""

from honeybee_energy_ph.load.ph_equipment import (
    PhDishwasherType, PhClothesWasherType, PhClothesDryerType, PhCookingType)

ph_default_equip = {
    'PhDishwasher': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,  # Year
            'energy_demand': 0,  # kWh
            'energy_demand_per_use': 1.1,
            'combined_energy_factor': 0,
            'capacity_type': 1,  # Standard
            'capacity': 12,
            'water_connection': PhDishwasherType("2-COLD WATER CONNECTION"),
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,  # Year
            'energy_demand': 269,  # kWh
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'capacity_type': 1,  # Standard
            'capacity': 12,
            'water_connection': PhDishwasherType("2-COLD WATER CONNECTION"),
        }
    },
    'PhClothesWasher': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,  # Year
            'energy_demand': 0,
            'energy_demand_per_use': 1.1,
            'combined_energy_factor': 0,
            'capacity': 0.1274,
            'modified_energy_factor': 2.7,
            'water_connection': PhClothesWasherType("2-COLD WATER CONNECTION"),
            'utilization_factor': 1.0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,  # Year
            'energy_demand': 120,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'capacity': 0.1274,
            'modified_energy_factor': 2.7,
            'water_connection': PhClothesWasherType("2-COLD WATER CONNECTION"),
            'utilization_factor': 1.0,
        }
    },
    'PhClothesDryer': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,  # CEF - Combined Energy Factor
            'energy_demand': 0,
            'energy_demand_per_use': 3.5,
            'combined_energy_factor': 3.93,
            'dryer_type': PhClothesDryerType("4-CONDENSATION DRYER"),
            'gas_consumption': 0,
            'gas_efficiency_factor': 2.67,
            'field_utilization_factor_type': 1,  # Timer controls
            'field_utilization_factor': 1.18,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,  # CEF - Combined Energy Factor
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 3.93,
            'dryer_type': PhClothesDryerType("4-CONDENSATION DRYER"),
            'gas_consumption': 0,
            'gas_efficiency_factor': 2.67,
            'field_utilization_factor_type': 1,  # Timer controls
            'field_utilization_factor': 1.18,
        }
    },
    'PhRefrigerator': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 4,  # PH case Units
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Day
            'energy_demand': 0,
            'energy_demand_per_use': 0.78,
            'combined_energy_factor': 0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 4,  # PH case Units
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Day
            'energy_demand': 1.0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
        }
    },
    'PhFreezer': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 4,  # PH case Units
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Day
            'energy_demand': 0,
            'energy_demand_per_use': 0.88,
            'combined_energy_factor': 0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 4,  # PH case Units
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Day
            'energy_demand': 0,
            'energy_demand_per_use': 0.88,
            'combined_energy_factor': 0,
        }
    },
    'PhFridgeFreezer': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 4,  # PH case Units
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Day
            'energy_demand': 0,
            'energy_demand_per_use': 1.0,
            'combined_energy_factor': 0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 4,  # PH case Units
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Day
            'energy_demand': 1.22,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
        }
    },
    'PhCooktop': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0.0,  # kWh/use
            'energy_demand_per_use': 0.25,  # kWh/use
            'combined_energy_factor': 0,
            'cooktop_type': PhCookingType("1-ELECTRICITY"),
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 1,  # PH case occupants
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0.2,  # kWh/use
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'cooktop_type': PhCookingType("1-ELECTRICITY"),
        }
    },
    'PhPhiusMEL': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 3,  # Bedroooms
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 3,  # Bedroooms
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
        }
    },
    'PhPhiusLightingInterior': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 6,  # PH case floor area
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 6,  # PH case floor area
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        }
    },
    'PhPhiusLightingExterior': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 6,  # PH case floor area
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 6,  # PH case floor area
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 1,  # Use
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        }
    },
    'PhPhiusLightingGarage': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 2,
            'quantity': 0,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 100,
            'energy_demand_per_use': 100,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 2,
            'quantity': 0,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 100,
            'energy_demand_per_use': 100,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        }
    },
    'PhCustomAnnualElectric': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 5,
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 5,
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        }
    },
    'PhCustomAnnualLighting': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 5,
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 5,
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        }
    },
    'PhCustomAnnualMEL': {
        'PHI': {
            'comment': 'default',
            'reference_quantity': 5,
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        },
        "PHIUS": {
            'comment': 'default',
            'reference_quantity': 5,
            'quantity': 1,
            'in_conditioned_space': True,
            'reference_energy_norm': 2,
            'energy_demand': 0,
            'energy_demand_per_use': 0,
            'combined_energy_factor': 0,
            'frac_high_efficiency': 1.0,
        }
    },
}
